==> vice_3.4.0.dfsg.orig.tar.xz: vice-3.4.0.dfsg/src/c64/cart/kingsoft.c <==
/*
 * kingsoft.h - Cartridge handling, Kingsoft "Business Basic" cart.
 *
 * Written by
 *  Groepaz/Hitmen <groepaz@gmx.net>
 *
 * This file is part of VICE, the Versatile Commodore Emulator.
 * See README for copyright notice.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307  USA.
 *
 */

#include "vice.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define CARTRIDGE_INCLUDE_SLOTMAIN_API
#include "c64cartsystem.h"
#undef CARTRIDGE_INCLUDE_SLOTMAIN_API
#include "c64mem.h"
#include "cartio.h"
#include "cartridge.h"
#include "comal80.h"
#include "export.h"
#include "lib.h"
#include "monitor.h"
#include "snapshot.h"
#include "types.h"
#include "util.h"
#include "crt.h"

#include "kingsoft.h"


/*
    Kingsoft Cartridge ("business basic" aka "s'more")

    - 24K ROM (3 8K Eproms)

    reading io1:
    - switches to 16k game mode
    - first eprom mapped to 8000 (ROML)
    - second eprom mapped to A000 (ROMH)

    writing io1:
    - switches to ultimax mode _only_:
      - if 0xc000 > address >= 0x8000
      - if address >= 0xe000
      (meaning 0x0000-0x7fff and 0xc000-0xdfff gives normal c64 ram/io)
    - first eprom mapped to 8000 (ROML)
    - third eprom mapped to e000 (ROMH)
*/

/* #define DEBUGKS */

#ifdef DEBUGKS
#define DBG(x)  printf x
#else
#define DBG(x)
#endif

static int mode = 0;

static void setmode(int m)
{
    mode = m;
    DBG(("KINGSOFT: mode: %s\n", mode ? "ultimax" : "16k"));
    if (mode) {
        cart_config_changed_slotmain(CMODE_ULTIMAX, CMODE_ULTIMAX, CMODE_READ);
    } else {
        cart_config_changed_slotmain(CMODE_16KGAME, CMODE_16KGAME, CMODE_READ);
    }
}

static uint8_t kingsoft_io1_read(uint16_t addr)
{
    setmode(0);
    return 0;
}

static void kingsoft_io1_store(uint16_t addr, uint8_t value)
{
    setmode(1);
}

static uint8_t kingsoft_io1_peek(uint16_t addr)
{
    return mode;
}

static int kingsoft_dump(void)
{
    mon_out("mode: %s\n", mode ? "ultimax" : "16k");
    return 0;
}

/* ---------------------------------------------------------------------*/

static io_source_t kingsoft_device = {
    CARTRIDGE_NAME_KINGSOFT, /* name of the device */
    IO_DETACH_CART,          /* use cartridge ID to detach the device when involved in a read-collision */
    IO_DETACH_NO_RESOURCE,   /* does not use a resource for detach */
    0xde00, 0xdeff, 0xff,    /* range for the device, address is ignored, reg:$de00, mirrors:$de01-$deff */
    0,                       /* read is never valid */
    kingsoft_io1_store,      /* store function */
    NULL,                    /* NO poke function */
    kingsoft_io1_read,       /* read function */
    kingsoft_io1_peek,       /* peek function */
    kingsoft_dump,           /* device state information dump function */
    CARTRIDGE_KINGSOFT,      /* cartridge ID */
    IO_PRIO_NORMAL,          /* normal priority, device read needs to be checked for collisions */
    0                        /* insertion order, gets filled in by the registration function */
};

static io_source_list_t *kingsoft_list_item = NULL;

static const export_resource_t export_res = {
    CARTRIDGE_NAME_KINGSOFT, 1, 1, &kingsoft_device, NULL, CARTRIDGE_KINGSOFT
};

/* ---------------------------------------------------------------------*/

/* ROML read - mapped to 8000 in 8k,16k,ultimax */
uint8_t kingsoft_roml_read(uint16_t addr)
{
    return roml_banks[(addr & 0x1fff)];
}

/* ROMH read - mapped to A000 in 16k, to E000 in ultimax */
uint8_t kingsoft_romh_read(uint16_t addr)
{
    return romh_banks[(addr & 0x1fff) + (mode << 13)];
}

/* ---------------------------------------------------------------------*/

void kingsoft_config_init(void)
{
    setmode(0);
}

void kingsoft_config_setup(uint8_t *rawcart)
{
    memcpy(&roml_banks[0x0000], &rawcart[0x0000], 0x2000);
    memcpy(&romh_banks[0x0000], &rawcart[0x2000], 0x2000);
    memcpy(&romh_banks[0x2000], &rawcart[0x4000], 0x2000);
    setmode(0);
}

/* ---------------------------------------------------------------------*/
static int kingsoft_common_attach(void)
{
    if (export_add(&export_res) < 0) {
        return -1;
    }
    kingsoft_list_item = io_source_register(&kingsoft_device);
    return 0;
}

int kingsoft_bin_attach(const char *filename, uint8_t *rawcart)
{
    if (util_file_load(filename, rawcart, 0x6000, UTIL_FILE_LOAD_SKIP_ADDRESS) < 0) {
        return -1;
    }
    return kingsoft_common_attach();
}

int kingsoft_crt_attach(FILE *fd, uint8_t *rawcart)
{
    crt_chip_header_t chip;

    while (1) {
        if (crt_read_chip_header(&chip, fd)) {
            break;
        }

        if (chip.start != 0x8000 || chip.size != 0x2000 || chip.bank > 3) {
            return -1;
        }

        if (crt_read_chip(rawcart, chip.bank << 13, &chip, fd)) {
            return -1;
        }
    }
    return kingsoft_common_attach();
}

void kingsoft_detach(void)
{
    export_remove(&export_res);
    io_source_unregister(kingsoft_list_item);
    kingsoft_list_item = NULL;
}

/* ---------------------------------------------------------------------*/

/* CARTKINGSOFT snapshot module format:

   type  | name | description
   --------------------------
   BYTE  | mode | mode
   ARRAY | ROML | 8192 BYTES of ROML data
   ARRAY | ROMH | 16384 BYTES of ROMH data
 */

static char snap_module_name[] = "CARTKINGSOFT";
#define SNAP_MAJOR   0
#define SNAP_MINOR   0

int kingsoft_snapshot_write_module(snapshot_t *s)
{
    snapshot_module_t *m;

    m = snapshot_module_create(s, snap_module_name, SNAP_MAJOR, SNAP_MINOR);

    if (m == NULL) {
        return -1;
    }

    if (0
        || (SMW_B(m, (uint8_t)mode) < 0)
        || (SMW_BA(m, roml_banks, 0x2000) < 0)
        || (SMW_BA(m, romh_banks, 0x4000) < 0)) {
        snapshot_module_close(m);
        return -1;
    }

    return snapshot_module_close(m);
}

int kingsoft_snapshot_read_module(snapshot_t *s)
{
    uint8_t vmajor, vminor;
    snapshot_module_t *m;

    m = snapshot_module_open(s, snap_module_name, &vmajor, &vminor);

    if (m == NULL) {
        return -1;
    }

    /* Do not accept versions higher than current */
    if (snapshot_version_is_bigger(vmajor, vminor, SNAP_MAJOR, SNAP_MINOR)) {
        snapshot_set_error(SNAPSHOT_MODULE_HIGHER_VERSION);
        goto fail;
    }

    if (0
        || (SMR_B_INT(m, &mode) < 0)
        || (SMR_BA(m, roml_banks, 0x2000) < 0)
        || (SMR_BA(m, romh_banks, 0x4000) < 0)) {
        goto fail;
    }

    snapshot_module_close(m);

    setmode(mode);
    return kingsoft_common_attach();

fail:
    snapshot_module_close(m);
    return -1;
}

==> vice_3.7.1+dfsg1.orig.tar.xz: vice-3.7.1+dfsg/src/c64/cart/kingsoft.c <==
/*
 * kingsoft.h - Cartridge handling, Kingsoft "Business Basic" cart.
 *
 * Written by
 *  Groepaz/Hitmen <groepaz@gmx.net>
 *
 * This file is part of VICE, the Versatile Commodore Emulator.
 * See README for copyright notice.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307  USA.
 *
 */

#include "vice.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define CARTRIDGE_INCLUDE_SLOTMAIN_API
#include "c64cartsystem.h"
#undef CARTRIDGE_INCLUDE_SLOTMAIN_API
#include "c64mem.h"
#include "cartio.h"
#include "cartridge.h"
#include "comal80.h"
#include "export.h"
#include "lib.h"
#include "monitor.h"
#include "snapshot.h"
#include "types.h"
#include "util.h"
#include "crt.h"

#include "kingsoft.h"


/*
    Kingsoft Cartridge ("business basic" aka "s'more")

    - 24K ROM (3 8K Eproms)

    reading io1:
    - switches to 16k game mode
    - first eprom mapped to 8000 (ROML)
    - second eprom mapped to A000 (ROMH)

    writing io1:
    - switches to ultimax mode _only_:
      - if 0xc000 > address >= 0x8000
      - if address >= 0xe000
      (meaning 0x0000-0x7fff and 0xc000-0xdfff gives normal c64 ram/io)
    - first eprom mapped to 8000 (ROML)
    - third eprom mapped to e000 (ROMH)
*/

/* #define DEBUGKS */

#ifdef DEBUGKS
#define DBG(x)  printf x
#else
#define DBG(x)
#endif

static int mode = 0;

static void setmode(int m)
{
    mode = m;
    DBG(("KINGSOFT: mode: %s\n", mode ? "ultimax" : "16k"));
    if (mode) {
        cart_config_changed_slotmain(CMODE_ULTIMAX, CMODE_ULTIMAX, CMODE_READ);
    } else {
        cart_config_changed_slotmain(CMODE_16KGAME, CMODE_16KGAME, CMODE_READ);
    }
}

static uint8_t kingsoft_io1_read(uint16_t addr)
{
    setmode(0);
    return 0;
}

static void kingsoft_io1_store(uint16_t addr, uint8_t value)
{
    setmode(1);
}

static uint8_t kingsoft_io1_peek(uint16_t addr)
{
    return mode;
}

static int kingsoft_dump(void)
{
    mon_out("mode: %s\n", mode ? "ultimax" : "16k");
    return 0;
}

/* ---------------------------------------------------------------------*/

static io_source_t kingsoft_device = {
    CARTRIDGE_NAME_KINGSOFT, /* name of the device */
    IO_DETACH_CART,          /* use cartridge ID to detach the device when involved in a read-collision */
    IO_DETACH_NO_RESOURCE,   /* does not use a resource for detach */
    0xde00, 0xdeff, 0xff,    /* range for the device, address is ignored, reg:$de00, mirrors:$de01-$deff */
    0,                       /* read is never valid */
    kingsoft_io1_store,      /* store function */
    NULL,                    /* NO poke function */
    kingsoft_io1_read,       /* read function */
    kingsoft_io1_peek,       /* peek function */
    kingsoft_dump,           /* device state information dump function */
    CARTRIDGE_KINGSOFT,      /* cartridge ID */
    IO_PRIO_NORMAL,          /* normal priority, device read needs to be checked for collisions */
    0                        /* insertion order, gets filled in by the registration function */
};

static io_source_list_t *kingsoft_list_item = NULL;

static const export_resource_t export_res = {
    CARTRIDGE_NAME_KINGSOFT, 1, 1, &kingsoft_device, NULL, CARTRIDGE_KINGSOFT
};

/* ---------------------------------------------------------------------*/

/* ROML read - mapped to 8000 in 8k,16k,ultimax */
uint8_t kingsoft_roml_read(uint16_t addr)
{
    return roml_banks[(addr & 0x1fff)];
}

/* ROMH read - mapped to A000 in 16k, to E000 in ultimax */
uint8_t kingsoft_romh_read(uint16_t addr)
{
    return romh_banks[(addr & 0x1fff) + (mode << 13)];
}

/* ---------------------------------------------------------------------*/

void kingsoft_config_init(void)
{
    setmode(0);
}

void kingsoft_config_setup(uint8_t *rawcart)
{
    memcpy(&roml_banks[0x0000], &rawcart[0x0000], 0x2000);
    memcpy(&romh_banks[0x0000], &rawcart[0x2000], 0x2000);
    memcpy(&romh_banks[0x2000], &rawcart[0x4000], 0x2000);
    setmode(0);
}

/* ---------------------------------------------------------------------*/
static int kingsoft_common_attach(void)
{
    if (export_add(&export_res) < 0) {
        return -1;
    }
    kingsoft_list_item = io_source_register(&kingsoft_device);
    return 0;
}

int kingsoft_bin_attach(const char *filename, uint8_t *rawcart)
{
    if (util_file_load(filename, rawcart, 0x6000, UTIL_FILE_LOAD_SKIP_ADDRESS) < 0) {
        return -1;
    }
    return kingsoft_common_attach();
}

int kingsoft_crt_attach(FILE *fd, uint8_t *rawcart)
{
    crt_chip_header_t chip;

    while (1) {
        if (crt_read_chip_header(&chip, fd)) {
            break;
        }

        if (chip.start != 0x8000 || chip.size != 0x2000 || chip.bank > 3) {
            return -1;
        }

        if (crt_read_chip(rawcart, chip.bank << 13, &chip, fd)) {
            return -1;
        }
    }
    return kingsoft_common_attach();
}

void kingsoft_detach(void)
{
    export_remove(&export_res);
    io_source_unregister(kingsoft_list_item);
    kingsoft_list_item = NULL;
}

/* ---------------------------------------------------------------------*/

/* CARTKINGSOFT snapshot module format:

   type  | name | description
   --------------------------
   BYTE  | mode | mode
   ARRAY | ROML | 8192 BYTES of ROML data
   ARRAY | ROMH | 16384 BYTES of ROMH data
 */

static const char snap_module_name[] = "CARTKINGSOFT";
#define SNAP_MAJOR   0
#define SNAP_MINOR   0

int kingsoft_snapshot_write_module(snapshot_t *s)
{
    snapshot_module_t *m;

    m = snapshot_module_create(s, snap_module_name, SNAP_MAJOR, SNAP_MINOR);

    if (m == NULL) {
        return -1;
    }

    if (0
        || (SMW_B(m, (uint8_t)mode) < 0)
        || (SMW_BA(m, roml_banks, 0x2000) < 0)
        || (SMW_BA(m, romh_banks, 0x4000) < 0)) {
        snapshot_module_close(m);
        return -1;
    }

    return snapshot_module_close(m);
}

int kingsoft_snapshot_read_module(snapshot_t *s)
{
    uint8_t vmajor, vminor;
    snapshot_module_t *m;

    m = snapshot_module_open(s, snap_module_name, &vmajor, &vminor);

    if (m == NULL) {
        return -1;
    }

    /* Do not accept versions higher than current */
    if (snapshot_version_is_bigger(vmajor, vminor, SNAP_MAJOR, SNAP_MINOR)) {
        snapshot_set_error(SNAPSHOT_MODULE_HIGHER_VERSION);
        goto fail;
    }

    if (0
        || (SMR_B_INT(m, &mode) < 0)
        || (SMR_BA(m, roml_banks, 0x2000) < 0)
        || (SMR_BA(m, romh_banks, 0x4000) < 0)) {
        goto fail;
    }

    snapshot_module_close(m);

    setmode(mode);
    return kingsoft_common_attach();

fail:
    snapshot_module_close(m);
    return -1;
}
